@@ -1,6 +1,7 @@
 /* UTF-8 decoding tests */
+/* Other string util tests too */
 
-#include "silcincludes.h"
+#include "silc.h"
 
 #define utf8fail(n, data, len)			\
 const unsigned char u##n[] = (data);		\
@@ -50,7 +51,7 @@ utf8fail(30, "\xf0\x20\xf9\x20\xfa\x20\xfb\x20", 8);
 
 int main(int argc, char **argv)
 {
-  bool success = FALSE;
+  SilcBool success = FALSE;
   unsigned char *s1, *s2, *s3, *s4;
   int l, opt;
 
@@ -67,6 +68,7 @@ int main(int argc, char **argv)
         case 'd':
           silc_log_debug(TRUE);
 	  silc_log_debug_hexdump(TRUE);
+	  silc_log_quick(TRUE);
           if (optarg)
             silc_log_set_debug_string(optarg);
 	  else
@@ -148,6 +150,18 @@ int main(int argc, char **argv)
   silc_free(s3);
   silc_free(s4);
 
+  /* Regex test */
+  SILC_LOG_DEBUG(("Simple regex test"));
+  s1 = "foo,bar,silc,com";
+  SILC_LOG_DEBUG(("Find 'silc' from %s", s1));
+  if (!silc_string_match(s1, "silc"))
+    goto err;
+  SILC_LOG_DEBUG(("Regex match"));
+  SILC_LOG_DEBUG(("Find 'foobar' from %s", s1));
+  if (silc_string_match(s1, "foobar"))
+    goto err;
+  SILC_LOG_DEBUG(("Regex not found (Ok)"));
+
   success = TRUE;
 
  err:
